@@ -8,63 +8,75 @@ import os
 import re
 
 from mercurial.i18n import _
+from mercurial.pycompat import open
 
 from mercurial import (
     error,
+    exthelper,
     httppeer,
     util,
-    wireproto,
+    wireprototypes,
+    wireprotov1peer,
+    wireprotov1server,
 )
 
-from . import (
-    lfutil,
-)
+from . import lfutil
 
 urlerr = util.urlerr
 urlreq = util.urlreq
 
-LARGEFILES_REQUIRED_MSG = ('\nThis repository uses the largefiles extension.'
-                           '\n\nPlease enable it in your Mercurial config '
-                           'file.\n')
+LARGEFILES_REQUIRED_MSG = (
+    b'\nThis repository uses the largefiles extension.'
+    b'\n\nPlease enable it in your Mercurial config '
+    b'file.\n'
+)
+
+eh = exthelper.exthelper()
 
 # these will all be replaced by largefiles.uisetup
-capabilitiesorig = None
 ssholdcallstream = None
 httpoldcallstream = None
 
+
 def putlfile(repo, proto, sha):
     '''Server command for putting a largefile into a repository's local store
     and into the user cache.'''
-    proto.redirect()
+    with proto.mayberedirectstdio() as output:
+        path = lfutil.storepath(repo, sha)
+        util.makedirs(os.path.dirname(path))
+        tmpfp = util.atomictempfile(path, createmode=repo.store.createmode)
 
-    path = lfutil.storepath(repo, sha)
-    util.makedirs(os.path.dirname(path))
-    tmpfp = util.atomictempfile(path, createmode=repo.store.createmode)
+        try:
+            for p in proto.getpayload():
+                tmpfp.write(p)
+            tmpfp._fp.seek(0)
+            if sha != lfutil.hexsha1(tmpfp._fp):
+                raise IOError(0, _(b'largefile contents do not match hash'))
+            tmpfp.close()
+            lfutil.linktousercache(repo, sha)
+        except IOError as e:
+            repo.ui.warn(
+                _(b'largefiles: failed to put %s into store: %s\n')
+                % (sha, e.strerror)
+            )
+            return wireprototypes.pushres(
+                1, output.getvalue() if output else b''
+            )
+        finally:
+            tmpfp.discard()
 
-    try:
-        proto.getfile(tmpfp)
-        tmpfp._fp.seek(0)
-        if sha != lfutil.hexsha1(tmpfp._fp):
-            raise IOError(0, _('largefile contents do not match hash'))
-        tmpfp.close()
-        lfutil.linktousercache(repo, sha)
-    except IOError as e:
-        repo.ui.warn(_('largefiles: failed to put %s into store: %s\n') %
-                     (sha, e.strerror))
-        return wireproto.pushres(1)
-    finally:
-        tmpfp.discard()
+    return wireprototypes.pushres(0, output.getvalue() if output else b'')
 
-    return wireproto.pushres(0)
 
 def getlfile(repo, proto, sha):
     '''Server command for retrieving a largefile from the repository-local
     cache or user cache.'''
     filename = lfutil.findfile(repo, sha)
     if not filename:
-        raise error.Abort(_('requested largefile %s not present in cache')
-                          % sha)
-    f = open(filename, 'rb')
+        raise error.Abort(
+            _(b'requested largefile %s not present in cache') % sha
+        )
+    f = open(filename, b'rb')
     length = os.fstat(f.fileno())[6]
 
     # Since we can't set an HTTP content-length header here, and
@@ -73,10 +85,12 @@ def getlfile(repo, proto, sha):
     # just send the length on the first line of the response, like the
     # ssh proto does for string responses.
     def generator():
-        yield '%d\n' % length
+        yield b'%d\n' % length
         for chunk in util.filechunkiter(f):
             yield chunk
-    return wireproto.streamres(gen=generator())
+
+    return wireprototypes.streamreslegacy(gen=generator())
+
 
 def statlfile(repo, proto, sha):
     '''Server command for checking if a largefile is present - returns '2\n' if
@@ -87,8 +101,9 @@ def statlfile(repo, proto, sha):
     server side.'''
     filename = lfutil.findfile(repo, sha)
     if not filename:
-        return '2\n'
-    return '0\n'
+        return wireprototypes.bytesresponse(b'2\n')
+    return wireprototypes.bytesresponse(b'0\n')
+
 
 def wirereposetup(ui, repo):
     class lfileswirerepository(repo.__class__):
@@ -97,41 +112,48 @@ def wirereposetup(ui, repo):
             # input file-like into a bundle before sending it, so we can't use
             # it ...
             if issubclass(self.__class__, httppeer.httppeer):
-                res = self._call('putlfile', data=fd, sha=sha,
-                    headers={'content-type':'application/mercurial-0.1'})
+                res = self._call(
+                    b'putlfile',
+                    data=fd,
+                    sha=sha,
+                    headers={r'content-type': r'application/mercurial-0.1'},
+                )
                 try:
-                    d, output = res.split('\n', 1)
+                    d, output = res.split(b'\n', 1)
                     for l in output.splitlines(True):
-                        self.ui.warn(_('remote: '), l) # assume l ends with \n
+                        self.ui.warn(_(b'remote: '), l)  # assume l ends with \n
                     return int(d)
                 except ValueError:
-                    self.ui.warn(_('unexpected putlfile response: %r\n') % res)
+                    self.ui.warn(_(b'unexpected putlfile response: %r\n') % res)
                     return 1
             # ... but we can't use sshrepository._call because the data=
             # argument won't get sent, and _callpush does exactly what we want
             # in this case: send the data straight through
             else:
                 try:
-                    ret, output = self._callpush("putlfile", fd, sha=sha)
-                    if ret == "":
-                        raise error.ResponseError(_('putlfile failed:'),
-                                output)
+                    ret, output = self._callpush(b"putlfile", fd, sha=sha)
+                    if ret == b"":
+                        raise error.ResponseError(
+                            _(b'putlfile failed:'), output
+                        )
                     return int(ret)
                 except IOError:
                     return 1
                 except ValueError:
                     raise error.ResponseError(
-                        _('putlfile failed (unexpected response):'), ret)
+                        _(b'putlfile failed (unexpected response):'), ret
+                    )
 
         def getlfile(self, sha):
             """returns an iterable with the chunks of the file with sha sha"""
-            stream = self._callstream("getlfile", sha=sha)
+            stream = self._callstream(b"getlfile", sha=sha)
             length = stream.readline()
             try:
                 length = int(length)
             except ValueError:
-                self._abort(error.ResponseError(_("unexpected response:"),
-                                                length))
+                self._abort(
+                    error.ResponseError(_(b"unexpected response:"), length)
+                )
 
             # SSH streams will block if reading more than length
             for chunk in util.filechunkiter(stream, limit=length):
@@ -141,13 +163,14 @@ def wirereposetup(ui, repo):
             if issubclass(self.__class__, httppeer.httppeer):
                 chunk = stream.read(1)
                 if chunk:
-                    self._abort(error.ResponseError(_("unexpected response:"),
-                                                    chunk))
+                    self._abort(
+                        error.ResponseError(_(b"unexpected response:"), chunk)
+                    )
 
-        @wireproto.batchable
+        @wireprotov1peer.batchable
         def statlfile(self, sha):
-            f = wireproto.future()
-            result = {'sha': sha}
+            f = wireprotov1peer.future()
+            result = {b'sha': sha}
             yield result, f
             try:
                 yield int(f.value)
@@ -160,30 +183,39 @@ def wirereposetup(ui, repo):
 
     repo.__class__ = lfileswirerepository
 
-# advertise the largefiles=serve capability
-def capabilities(repo, proto):
-    '''Wrap server command to announce largefile server capability'''
-    return capabilitiesorig(repo, proto) + ' largefiles=serve'
 
-def heads(repo, proto):
+# advertise the largefiles=serve capability
+@eh.wrapfunction(wireprotov1server, b'_capabilities')
+def _capabilities(orig, repo, proto):
+    '''announce largefile server capability'''
+    caps = orig(repo, proto)
+    caps.append(b'largefiles=serve')
+    return caps
+
+
+def heads(orig, repo, proto):
     '''Wrap server command - largefile capable clients will know to call
     lheads instead'''
     if lfutil.islfilesrepo(repo):
-        return wireproto.ooberror(LARGEFILES_REQUIRED_MSG)
-    return wireproto.heads(repo, proto)
+        return wireprototypes.ooberror(LARGEFILES_REQUIRED_MSG)
+
+    return orig(repo, proto)
+
 
 def sshrepocallstream(self, cmd, **args):
-    if cmd == 'heads' and self.capable('largefiles'):
-        cmd = 'lheads'
-    if cmd == 'batch' and self.capable('largefiles'):
-        args['cmds'] = args['cmds'].replace('heads ', 'lheads ')
+    if cmd == b'heads' and self.capable(b'largefiles'):
+        cmd = b'lheads'
+    if cmd == b'batch' and self.capable(b'largefiles'):
+        args[r'cmds'] = args[r'cmds'].replace(b'heads ', b'lheads ')
     return ssholdcallstream(self, cmd, **args)
 
-headsre = re.compile(r'(^|;)heads\b')
+
+headsre = re.compile(br'(^|;)heads\b')
+
 
 def httprepocallstream(self, cmd, **args):
-    if cmd == 'heads' and self.capable('largefiles'):
-        cmd = 'lheads'
-    if cmd == 'batch' and self.capable('largefiles'):
-        args['cmds'] = headsre.sub('lheads', args['cmds'])
+    if cmd == b'heads' and self.capable(b'largefiles'):
+        cmd = b'lheads'
+    if cmd == b'batch' and self.capable(b'largefiles'):
+        args[r'cmds'] = headsre.sub(b'lheads', args[r'cmds'])
     return httpoldcallstream(self, cmd, **args)
